@@ -9,15 +9,19 @@ from .parameters import set_params
 import os
 
 initialized = False
+
+
 def initialize():
     global ws, initialized
     initialized = True
     session_id = os.environ.get('SESSION_ID')
     # ws_host = os.environ.get('WS_HOST', 'ws://localhost:8080')
-    ws_host = os.environ.get('WS_HOST', 'wss://hackerforms-broker.abstra.cloud')
+    ws_host = os.environ.get(
+        'WS_HOST', 'wss://hackerforms-broker.abstra.cloud')
 
     # frontend_host = os.environ.get('FRONTEND_HOST', 'http://localhost:8001')
-    frontend_host = os.environ.get('FRONTEND_HOST', 'https://console.abstracloud.com')
+    frontend_host = os.environ.get(
+        'FRONTEND_HOST', 'https://console.abstracloud.com')
 
     if session_id == None:
         ws = create_connection(f'{ws_host}/lib')
@@ -26,22 +30,22 @@ def initialize():
     else:
         ws = create_connection(f'{ws_host}/lib?sessionId={session_id}')
 
-
-    start = { "type": None }
+    start = {"type": None}
     while start["type"] != 'start':
         start = receive()
 
     set_params(start["params"])
-    
 
 
 def send(data):
-    if not initialized: return
+    if not initialized:
+        return
     ws.send(serialize(data))
 
 
 def receive(path: str = ''):
-    if not initialized: return
+    if not initialized:
+        return
     data = deserialize(ws.recv())
 
     if data['type'] == 'keep-alive':
@@ -54,7 +58,8 @@ def receive(path: str = ''):
 
 @atexit.register
 def close():
-    if not initialized: return
+    if not initialized:
+        return
     send({
         'type': 'program:end',
         'exitCode': hooks.exit_code,
